# frozen_string_literal: true

require_relative './html'

class Rubyoshka
  # Markup extensions 
  module HTML
    # Emits the p tag (overrides Object#p)
    # @param text [String] text content of tag
    # @param props [Hash] tag attributes
    # @para block [Proc] nested HTML block
    # @return [void]
    def p(text = nil, **props, &block)
      method_missing(:p, text, **props, &block)
    end
  
    S_HTML5_DOCTYPE = '<!DOCTYPE html>'
  
    # Emits an HTML5 doctype tag and an html tag with the given block
    # @param block [Proc] nested HTML block
    # @return [void]
    def html5(&block)
      @buffer << S_HTML5_DOCTYPE
      self.html(&block)
    end

    def link_stylesheet(href, custom_attributes = nil)
      attributes = {
        rel: 'stylesheet',
        href: href
      }
      if custom_attributes
        attributes = custom_attributes.merge(attributes)
      end
      link(**attributes)
    end
  end
end
